#include <unistd.h>
#include <stdint.h>
#include <stddef.h>

/* TI Driver Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/timer/GPTimerCC26XX.h>
#include <ti/devices/cc13x0/driverlib/ioc.h>
#include <ti/sysbios/knl/Task.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/hal/Hwi.h>
#include <ti/sysbios/knl/Clock.h>
#include "ti_drivers_config.h"

GPTimerCC26XX_Handle hTimer;
GPTimerCC26XX_Params timerParams;

volatile uint32_t pulseTimePrev = 0;
volatile uint32_t pulseTimeCurr = 0;
volatile uint32_t pulseCount = 0;
volatile bool counting = false;

/* Timer Capture Interrupt Callback */
void timerCallback(GPTimerCC26XX_Handle handle, GPTimerCC26XX_IntMask interruptMask) {
    if (!counting) return;  // Ignore if not in counting phase

    pulseTimePrev = pulseTimeCurr;
    pulseTimeCurr = GPTimerCC26XX_getValue(handle);

    System_printf("⚡ Interrupt Triggered! Timer Value: %u\n", pulseTimeCurr);
    System_flush();

    if (pulseTimePrev > 0) {
        uint32_t pulseWidth = pulseTimeCurr - pulseTimePrev;

//        if (pulseWidth >= 480 && pulseWidth <= 550) {  // ✅ 11µs pulse width (48MHz clock)
            pulseCount++;
//        }
    }
}

/* Initialize and Configure GPTimer for Pulse Capture */
void initPulseCapture() {
    GPTimerCC26XX_Params_init(&timerParams);
    timerParams.mode = GPT_MODE_EDGE_TIME;  // ✅ Use Capture Mode
    timerParams.width = GPT_CONFIG_16BIT;
    timerParams.direction = GPTimerCC26XX_DIRECTION_UP;
    timerParams.debugStallMode = GPTimerCC26XX_DEBUG_STALL_OFF;

    hTimer = GPTimerCC26XX_open(CONFIG_GPTIMER_0, &timerParams);
    if (hTimer == NULL) {
        System_printf("❌ Error: Failed to open GPTimer\n");
        System_flush();
        return;
    }

    // ✅ Properly Map IOID_13 to GPTimer
    GPTimerCC26XX_PinMux pinMux = GPTimerCC26XX_getPinMux(hTimer);
    IOCPortConfigureSet(IOID_13, pinMux, IOC_STD_INPUT);

    // ✅ Register Timer Interrupt
    GPTimerCC26XX_registerInterrupt(hTimer, timerCallback, GPT_INT_CAPTURE);
    System_printf("✅ Timer interrupt registered!\n");
    System_flush();

    // ✅ Enable NVIC Interrupt
    Hwi_enableInterrupt(INT_AON_GPIO_EDGE);
    System_printf("✅ NVIC Interrupt Enabled!\n");
    System_flush();

    // ✅ Capture Both Rising and Falling Edges
    GPTimerCC26XX_setCaptureEdge(hTimer, GPTimerCC26XX_NEG_EDGE);

    // ✅ Start Timer
    GPTimerCC26XX_start(hTimer);
    System_printf("✅ GPTimer started!\n");
    System_flush();
}

/* Start Counting Pulses After 50ms HIGH Phase */
void startPulseCounting() {
    pulseCount = 0;  // Reset count

    System_printf("🔄 Waiting for 50ms HIGH phase...\n");
    System_flush();
    Task_sleep(50 * 1000 / Clock_tickPeriod);  // ✅ Wait for 50ms HIGH Phase

    System_printf("✅ 50ms HIGH phase ended. Starting 54ms pulse count...\n");
    System_flush();

    counting = true;  // ✅ Enable pulse counting
    Task_sleep(54 * 1000 / Clock_tickPeriod);  // ✅ Count pulses for 54ms

    counting = false;  // ✅ Stop counting
    System_printf("✅ Pulse Count Complete: %u pulses\n", pulseCount);
    System_flush();
}

void *mainThread(void *arg0) {
    GPIO_init();
    initPulseCapture();  // ✅ Use GPTimer in Capture Mode
//    GPIO_setConfig(IOID_14, GPIO_CFG_OUTPUT | GPIO_CFG_OUT_HIGH ); // Configure as output, start low

//    GPIO_write(IOID_14, 0);

    while (1) {
//        GPIO_write(IOID_14, 1);
        startPulseCounting();  // ✅ Wait 50ms HIGH + Count Pulses for 54ms
//        GPIO_write(IOID_14, 0);
        sleep(1);
    }
}
